@@ -1,7 +1,7 @@
 package Perinci::CmdLine::Base;
 
-our $DATE = '2015-01-15'; # DATE
-our $VERSION = '0.79'; # VERSION
+our $DATE = '2015-01-17'; # DATE
+our $VERSION = '0.80'; # VERSION
 
 use 5.010001;
 use strict;
@@ -755,6 +755,23 @@ sub parse_argv {
     $self->_parse_argv2($r);
 }
 
+sub __gen_iter {
+    my ($fh, $type) = @_;
+    return sub {
+        # XXX this will be configurable later. currently by default reading
+        # binary is per-64k while reading string is line-by-line.
+        local $/ = \(64*1024) if $type eq 'buf';
+
+        state $eof;
+        return undef if $eof;
+        my $l = <$fh>;
+        unless (defined $l) {
+            $eof++; return undef;
+        }
+        $l;
+    };
+}
+
 # parse cmdline_src argument spec properties for filling argument value from
 # file and/or stdin. currently does not support argument submetadata.
 sub parse_cmdline_src {
@@ -834,7 +851,8 @@ sub parse_cmdline_src {
                             $r->{args}{$an} ne '-';
                     #$log->trace("Getting argument '$an' value from stdin ...");
                     $r->{args}{$an} = $r->{stream_arg} ?
-                        \*STDIN : $is_ary ? [<STDIN>] : do {local $/;<STDIN>};
+                        __gen_iter(\*STDIN, $type) : $is_ary ? [<STDIN>] :
+                            do {local $/; ~~<STDIN>};
                     $r->{args}{"-cmdline_src_$an"} = 'stdin';
                 } elsif ($src eq 'stdin_or_files') {
                     # push back argument value to @ARGV so <> can work to slurp
@@ -852,7 +870,8 @@ sub parse_cmdline_src {
                     }
 
                     $r->{args}{$an} = $r->{stream_arg} ?
-                        \*ARGV : $is_ary ? [<>] : do { local $/; <> };
+                        __gen_iter(\*ARGV, $type) : $is_ary ? [<>] :
+                            do {local $/; ~~<>};
                     $r->{args}{"-cmdline_src_$an"} = 'stdin_or_files';
                 } elsif ($src eq 'file') {
                     unless (exists $r->{args}{$an}) {
@@ -874,7 +893,8 @@ sub parse_cmdline_src {
                                  ": $!"];
                     }
                     $r->{args}{$an} = $r->{stream_arg} ?
-                        $fh : $is_ary ? [<$fh>] : do { local $/; <$fh> };
+                        __gen_iter($fh, $type) : $is_ary ? [<$fh>] :
+                            do { local $/; ~~<$fh> };
                     $r->{args}{"-cmdline_src_$an"} = 'file';
                     $r->{args}{"-cmdline_srcfilename_$an"} = $fname;
                 }
@@ -928,33 +948,21 @@ sub select_output_handle {
 sub display_result {
     my ($self, $r) = @_;
 
+    my $meta = $r->{meta};
     my $res = $r->{res};
     my $fres = $r->{fres};
     my $resmeta = $res->[3] // {};
 
     my $handle = $r->{output_handle};
 
-    use experimental 'smartmatch';
-    if ($resmeta->{stream}) {
+    if ($resmeta->{stream} // $meta->{result}{stream}) {
         my $x = $res->[2];
-        if (ref($x) eq 'GLOB') {
-            while (!eof($x)) {
-                while(<$x>) { print $handle $_ }
-            }
-        } elsif (blessed($x) && $x->can('getline') && $x->can('eof')) {
-            # IO::Handle-like object
-            while (!$x->eof) {
-                print $handle $x->getline;
-            }
-        } elsif (ref($x) eq 'ARRAY') {
-            # tied array
-            while (~~(@$x) > 0) {
-                $log->tracef("[pericmd] Running hook_format_row ...");
-                print $handle $self->hook_format_row($r, shift(@$x));
+        if (ref($x) eq 'CODE') {
+            while (defined(my $l = $x->())) {
+                print $l;
             }
         } else {
-            die [500, "Invalid stream in result (not a glob/IO::Handle-like ".
-                     "object/(tied) array)\n"];
+            die [500, "Invalid stream in result (not a coderef)"];
         }
     } else {
         print $handle $fres;
@@ -1036,7 +1044,7 @@ sub run {
   FORMAT:
     if ($r->{res}[3]{'cmdline.skip_format'}) {
         $r->{fres} = $r->{res}[2];
-    } elsif ($r->{res}[3]{stream}) {
+    } elsif ($r->{res}[3]{stream} // $r->{meta}{result}{stream}) {
         # stream will be formatted as displayed by display_result()
     } else {
         $log->tracef("[pericmd] Running hook_format_result ...");
@@ -1080,7 +1088,7 @@ Perinci::CmdLine::Base - Base class for Perinci::CmdLine{,::Lite}
 
 =head1 VERSION
 
-This document describes version 0.79 of Perinci::CmdLine::Base (from Perl distribution Perinci-CmdLine-Lite), released on 2015-01-15.
+This document describes version 0.80 of Perinci::CmdLine::Base (from Perl distribution Perinci-CmdLine-Lite), released on 2015-01-17.
 
 =head1 DESCRIPTION
 
@@ -1869,7 +1877,7 @@ Please visit the project's homepage at L<https://metacpan.org/release/Perinci-Cm
 
 =head1 SOURCE
 
-Source repository is at L<https://github.com/perlancar/perl-Perinci-CmdLine-Lite>.
+Source repository is at L<https://github.com/sharyanto/perl-Perinci-CmdLine-Lite>.
 
 =head1 BUGS
 
